#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Compatibility to Python's `urllib`:mod: standard module.

.. deprecated:: 2.0.0

'''

from urllib import error, parse, request, response, robotparser  # noqa
